@@ -15,7 +15,7 @@ use List::MoreUtils                 qw(any firstval);
 
 use Mojolicious::Plugin::Vparam::Address;
 
-our $VERSION = '1.5';
+our $VERSION = '1.6';
 
 =encoding utf-8
 
@@ -117,12 +117,35 @@ Get many parameters as hash. By default all parameters are required.
 
 =head2 vsort
 
-Like I<vparams> but add some keys to simple use with tables.
+Like I<vparams> but add some keys to simple use with tables. Example:
 
-    %filters = $self->vsort(
+    # HTML - table with controls and filters
+    Order by:
+    <select name="oby">
+        <option value="0">Name</option>
+        <option value="1">Date</option>
+    </select>
+    Order direction:
+    <select name="ods">
+        <option value="asc">Low to High</option>
+        <option value="desc">High to Low</option>
+    </select>
+    Count per page:
+    <select name="rws">
+        <option value="10">10</option>
+        <option value="100">100</option>
+    </select>
+    Filter by name:
+    <input type="text" name="name" value="">
+    Any other filters ...
+
+
+    # Controller
+    %params = $self->vsort(
         -sort       => ['name', 'date', ...],
 
         # next as vparams
+        name        => 'text',
         ...
     );
 
@@ -147,9 +170,14 @@ If you set undef then parameter is not apply.
 
 Column number for sorting. Default: 1 - in many cases first database
 column is primary key.
-If you set undef then parameter is not apply.
 
 You can set different name by I<vsort_oby> config parameter.
+If you set undef then parameter is not apply.
+
+Value of B<oby> The value will be automatically mapped to the column name
+using the L</-sort> attribute.
+Also, the value will be checked for proper mapping.
+So you do not need to worry about it.
 
 =item ods
 
@@ -825,8 +853,37 @@ Then true and value is not passed validation don`t set verrors.
         param2      => 'str',
     );
 
+=item skip
+
+So as not to smear the validation code you can use the I<skip> parameter
+to skip on the condition.
+This attribute is useful for controlling access to the form fields.
+
+    # This example don`t get param1 in production mode.
+
+    # HTML
+    % unless( $self->app->mode eq 'production' ) {
+        %= number_field 'param1'
+    % }
+
+    # Simple flag
+    $param1 = $self->vparam(
+        param1      => 'int', skip => $self->app->mode eq 'production',
+    );
+
+    # Same as by use sub.
+    $param1 = $self->vparam(
+        param1      => 'int', skip => sub { $_[0]->app->mode eq 'production' },
+    );
+
+If you use sub then first parameter is controller.
+
+This attribute
+
 =back
 
+=cut
+
 =head1 RESERVED ATTRIBUTES
 
 =over
@@ -952,8 +1009,8 @@ Check maximum length in utf8.
 
 sub _size($$$) {
     my ($value, $min, $max) = @_;
-    return 'Value is not defined'       unless defined $_[0];
-    return 'Value is not set'           unless length  $_[0];
+    return 'Value is not defined'       unless defined $value;
+    return 'Value is not set'           unless length  $value;
     return sprintf "Value should not be less than %s", $min
         unless $min <= length $value;
     return sprintf "Value should not be longer than %s", $max
@@ -1126,7 +1183,10 @@ sub register {
 
     $conf->{password_min}   //= 8;
 
-    $conf->{mojo_validator} //= 1;
+    # Enable Mojolicious::Validator::Validation integration if available
+    $conf->{mojo_validator} //=
+        version->new($Mojolicious::VERSION) < version->new(4.42) ? 0 : 1;
+
 
     $conf->{types} = {
         # Numbers
@@ -1371,6 +1431,17 @@ sub register {
                 $attr{type}     = $def;
             }
 
+            # Skip
+            if( exists $attr{skip} ) {
+                if( 'CODE' eq ref $attr{skip} ) {
+                    # Skip by sub result
+                    next if $attr{skip}->($self);
+                } elsif( $attr{skip} ) {
+                    # Skip by flag
+                    next;
+                }
+            }
+
             # Set default optional
             $attr{optional} = $optional unless defined $attr{optional};
 
@@ -1417,83 +1488,74 @@ sub register {
             for my $index ( 0 .. $#input ) {
                 my $in = my $out = $input[$index];
 
-                if( defined $in ) {
-                    $out = $in;
+                $out = $in;
 
-                    # Apply pre filter
-                    $out = $attr{pre}->( $self, $out )    if $attr{pre};
+                # Apply pre filter
+                $out = $attr{pre}->( $self, $out )    if $attr{pre};
 
-                    # Apply validator
-                    if( $attr{valid} ) {
-                        if( my $error = $attr{valid}->($self, $out)  ) {
-                            # Set default value if error
-                            $out = $attr{default};
+                # Apply validator
+                if( $attr{valid} ) {
+                    if( my $error = $attr{valid}->($self, $out)  ) {
+                        # Set default value if error
+                        $out = $attr{default};
 
-                            # Default value always supress error
-                            $error = 0 if defined $attr{default};
-                            # Disable error on optional
-                            if( $attr{optional} ) {
-                                # Only if input param not set
-                                $error = 0 if not defined $in;
-                                $error = 0 if defined($in) and $in =~ m{^\s*$};
-                            }
-
-                            $self->verror(
-                                $name,
-                                %attr,
-                                index   => $index,
-                                in      => $in,
-                                out     => $out,
-                                message => $error,
-                            ) if $error;
+                        # Default value always supress error
+                        $error = 0 if defined $attr{default};
+                        # Disable error on optional
+                        if( $attr{optional} ) {
+                            # Only if input param not set
+                            $error = 0 if not defined $in;
+                            $error = 0 if defined($in) and $in =~ m{^\s*$};
                         }
+
+                        $self->verror(
+                            $name,
+                            %attr,
+                            index   => $index,
+                            in      => $in,
+                            out     => $out,
+                            message => $error,
+                        ) if $error;
                     }
+                }
 
-                    # Apply post filter
-                    $out = $attr{post}->( $self, $out )   if $attr{post};
+                # Hack for bool values:
+                # HTML forms do not transmit if checkbox off
+                $out = $attr{default}
+                    if $attr{type} && $attr{type} eq 'bool' and not defined $in;
 
-                    for my $key ( keys %attr ) {
-                        # Skip not filters
-                        next if $key eq 'type';
-                        next if $key eq 'pre';
-                        next if $key eq 'valid';
-                        next if $key eq 'post';
-                        next if $key eq 'optional';
-                        next if $key eq 'default';
-                        next if $key eq 'array';
+                # Apply post filter
+                $out = $attr{post}->( $self, $out )   if $attr{post};
 
-                        # Skip unknown attribute
-                        next unless $conf->{filters}{ $key };
+                for my $key ( keys %attr ) {
+                    # Skip unknown attribute
+                    next unless $conf->{filters}{ $key };
 
-                        my $error = $conf->{filters}{ $key }->(
-                            $self, $out, $attr{ $key }
-                        );
-                        if( $error ) {
-                            # Set default value if error
-                            $out = $attr{default};
+                    my $error = $conf->{filters}{ $key }->(
+                        $self, $out, $attr{ $key }
+                    );
+                    if( $error ) {
+                        # Set default value if error
+                        $out = $attr{default};
 
-                            # Default value always supress error
-                            $error = 0 if defined $attr{default};
-                            # Disable error on optional
-                            if( $attr{optional} ) {
-                                # Only if input param not set
-                                $error = 0 if not defined $in;
-                                $error = 0 if defined($in) and $in =~ m{^\s*$};
-                            }
-
-                            $self->verror(
-                                $name,
-                                %attr,
-                                index   => $index,
-                                in      => $in,
-                                out     => $out,
-                                message => $error,
-                            ) if $error;
+                        # Default value always supress error
+                        $error = 0 if defined $attr{default};
+                        # Disable error on optional
+                        if( $attr{optional} ) {
+                            # Only if input param not set
+                            $error = 0 if not defined $in;
+                            $error = 0 if defined($in) and $in =~ m{^\s*$};
                         }
+
+                        $self->verror(
+                            $name,
+                            %attr,
+                            index   => $index,
+                            in      => $in,
+                            out     => $out,
+                            message => $error,
+                        ) if $error;
                     }
-                } else {
-                    $out = $attr{default};
-                    $out = $attr{post}->( $self, $out )   if $attr{post};
                 }
 
                 # Запишем полученное значение
